@@ -46,9 +46,7 @@
 #include "dmxvisual.h"
 #include "dmxinput.h"
 #include "dmxextension.h"
-#ifdef RENDER
 #include "dmxpict.h"
-#endif
 
 #include "windowstr.h"
 
@@ -57,14 +55,12 @@ static void dmxDoChangeWindowAttributes(WindowPtr pWindow,
 					unsigned long *mask,
 					XSetWindowAttributes *attribs);
 
-#ifdef SHAPE
 static void dmxDoSetShape(WindowPtr pWindow);
-#endif
 
 /** Initialize the private area for the window functions. */
 Bool dmxInitWindow(ScreenPtr pScreen)
 {
-    if (!dixRequestPrivate(dmxWinPrivateKey, sizeof(dmxWinPrivRec)))
+    if (!dixRegisterPrivateKey(&dmxWinPrivateKeyRec, PRIVATE_WINDOW, sizeof(dmxWinPrivRec)))
 	return FALSE;
 
     return TRUE;
@@ -118,7 +114,7 @@ Window dmxCreateRootWindow(WindowPtr pWindow)
 }
 
 /** Change the location and size of the "screen" window.  Called from
- *  #dmxReconfigureScreenWindow(). */
+ *  #dmxConfigureScreenWindow(). */
 void dmxResizeScreenWindow(ScreenPtr pScreen,
 			   int x, int y, int w, int h)
 {
@@ -141,7 +137,7 @@ void dmxResizeScreenWindow(ScreenPtr pScreen,
 }
 
 /** Change the location and size of the "root" window.  Called from
- *  #dmxReconfigureRootWindow(). */
+ *  #dmxConfigureRootWindow. */
 void dmxResizeRootWindow(WindowPtr pRoot,
 			 int x, int y, int w, int h)
 {
@@ -289,12 +285,8 @@ void dmxCreateAndRealizeWindow(WindowPtr pWindow, Bool doSync)
 
     pWinPriv->window = dmxCreateNonRootWindow(pWindow);
     if (pWinPriv->restacked) dmxDoRestackWindow(pWindow);
-#ifdef SHAPE
     if (pWinPriv->isShaped) dmxDoSetShape(pWindow);
-#endif
-#ifdef RENDER
     if (pWinPriv->hasPict) dmxCreatePictureList(pWindow);
-#endif
     if (pWinPriv->mapped) XMapWindow(dmxScreen->beDisplay,
 				      pWinPriv->window);
     if (doSync) dmxSync(dmxScreen, False);
@@ -323,12 +315,8 @@ Bool dmxCreateWindow(WindowPtr pWindow)
     pWinPriv->mapped     = FALSE;
     pWinPriv->restacked  = FALSE;
     pWinPriv->attribMask = 0;
-#ifdef SHAPE
     pWinPriv->isShaped   = FALSE;
-#endif
-#ifdef RENDER
     pWinPriv->hasPict    = FALSE;
-#endif
 #ifdef GLXEXT
     pWinPriv->swapGroup  = NULL;
     pWinPriv->barrier    = 0;
@@ -411,10 +399,8 @@ Bool dmxDestroyWindow(WindowPtr pWindow)
 
     DMX_UNWRAP(DestroyWindow, dmxScreen, pScreen);
 
-#ifdef RENDER
     /* Destroy any picture list associated with this window */
     needSync |= dmxDestroyPictureList(pWindow);
-#endif
 
     /* Destroy window on back-end server */
     needSync |= dmxBEDestroyWindow(pWindow);
@@ -425,10 +411,9 @@ Bool dmxDestroyWindow(WindowPtr pWindow)
 	pWinPriv->windowDestroyed(pWindow);
 #endif
 
-#if 0
     if (pScreen->DestroyWindow)
 	ret = pScreen->DestroyWindow(pWindow);
-#endif
+
     DMX_WRAP(DestroyWindow, dmxDestroyWindow, dmxScreen, pScreen);
 
     return ret;
@@ -942,7 +927,6 @@ void dmxChangeBorderWidth(WindowPtr pWindow, unsigned int width)
     DMX_WRAP(ChangeBorderWidth, dmxChangeBorderWidth, dmxScreen, pScreen);
 }
 
-#ifdef SHAPE
 static void dmxDoSetShape(WindowPtr pWindow)
 {
     ScreenPtr       pScreen = pWindow->drawable.pScreen;
@@ -956,9 +940,9 @@ static void dmxDoSetShape(WindowPtr pWindow)
 
     /* First, set the bounding shape */
     if (wBoundingShape(pWindow)) {
-	pBox = REGION_RECTS(wBoundingShape(pWindow));
-	nRect = nBox = REGION_NUM_RECTS(wBoundingShape(pWindow));
-	pRectFirst = pRect = xalloc(nRect * sizeof(*pRect));
+	pBox = RegionRects(wBoundingShape(pWindow));
+	nRect = nBox = RegionNumRects(wBoundingShape(pWindow));
+	pRectFirst = pRect = malloc(nRect * sizeof(*pRect));
 	while (nBox--) {
 	    pRect->x      = pBox->x1;
 	    pRect->y      = pBox->y1;
@@ -971,7 +955,7 @@ static void dmxDoSetShape(WindowPtr pWindow)
 				ShapeBounding, 0, 0,
 				pRectFirst, nRect,
 				ShapeSet, YXBanded);
-	xfree(pRectFirst);
+	free(pRectFirst);
     } else {
 	XShapeCombineMask(dmxScreen->beDisplay, pWinPriv->window,
 			  ShapeBounding, 0, 0, None, ShapeSet);
@@ -979,9 +963,9 @@ static void dmxDoSetShape(WindowPtr pWindow)
 
     /* Next, set the clip shape */
     if (wClipShape(pWindow)) {
-	pBox = REGION_RECTS(wClipShape(pWindow));
-	nRect = nBox = REGION_NUM_RECTS(wClipShape(pWindow));
-	pRectFirst = pRect = xalloc(nRect * sizeof(*pRect));
+	pBox = RegionRects(wClipShape(pWindow));
+	nRect = nBox = RegionNumRects(wClipShape(pWindow));
+	pRectFirst = pRect = malloc(nRect * sizeof(*pRect));
 	while (nBox--) {
 	    pRect->x      = pBox->x1;
 	    pRect->y      = pBox->y1;
@@ -994,7 +978,7 @@ static void dmxDoSetShape(WindowPtr pWindow)
 				ShapeClip, 0, 0,
 				pRectFirst, nRect,
 				ShapeSet, YXBanded);
-	xfree(pRectFirst);
+	free(pRectFirst);
     } else {
 	XShapeCombineMask(dmxScreen->beDisplay, pWinPriv->window,
 			  ShapeClip, 0, 0, None, ShapeSet);
@@ -1007,7 +991,7 @@ static void dmxDoSetShape(WindowPtr pWindow)
 }
 
 /** Set shape of \a pWindow on the back-end server. */
-void dmxSetShape(WindowPtr pWindow)
+void dmxSetShape(WindowPtr pWindow, int kind)
 {
     ScreenPtr       pScreen = pWindow->drawable.pScreen;
     DMXScreenInfo  *dmxScreen = &dmxScreens[pScreen->myNum];
@@ -1016,7 +1000,7 @@ void dmxSetShape(WindowPtr pWindow)
     DMX_UNWRAP(SetShape, dmxScreen, pScreen);
 #if 1
     if (pScreen->SetShape)
-	pScreen->SetShape(pWindow);
+	pScreen->SetShape(pWindow, kind);
 #endif
 
     if (pWinPriv->window) {
@@ -1029,4 +1013,3 @@ void dmxSetShape(WindowPtr pWindow)
 
     DMX_WRAP(SetShape, dmxSetShape, dmxScreen, pScreen);
 }
-#endif
